
/**
 * imageBG
 *  the entire background div. Should have background-image:url(image url here)
 *
 * @param left         the absoloute left offset of the div in pixels
 * @param top          the absoloute position offset of the top in pixels
 * @param height      the height of the image (and div) in pixels
 * @param width       the width of the image (and div) in pixels
 * @param img_id      the id attribute of the div
 */
 function imageBG(top, left, height, width, img_id) {
	left = findInt(left);
	top = findInt(top);
	height = findInt(height);
	width = findInt(width);
  
	this.id = img_id;

  //The Rect that represents the whole div
  //Set the points absolutely relative to the page  
	this.imageBox = new Rect();
  this.SetOffset(top, left, height, width);
    
  this.canvases = new Array();              //Each canvas represents one shape
  this.counter = 1;                                //Count of the canvases 
  this.currently_active = 0;                    //Nothing currently active	
  
}

imageBG.prototype.ChangeCanvas = function(container_id, ui) {
	var num = findInt(container_id);
	this.canvases[num].ChangeMe( );	
};

imageBG.prototype.SetOffset = function(top, left, height, width) {
  left = findInt(left);
  top= findInt(top);
  height = findInt(height);
  width = findInt(width);
  
	this.imageBox.SetPoints(left, left + width, top, top + height);
	this.maxW = this.imageBox.GetWidth()/2;                                           //max width
	this.maxB = this.imageBox.GetBottomY() - this.imageBox.GetTopY(); //max down
	this.maxR = this.imageBox.GetRightX() - this.imageBox.GetLeftX();    //max right
	this.maxH= this.imageBox.GetHeight();                                              //max height
	this.minL= this.imageBox.GetLeftX();                                                  //min left
};

imageBG.prototype.ChangeLine= function ( num_or_id, int_width ) {  
	var num = -1;
	if (num_or_id) {
		var num = findInt(num_or_id);
    this.canvases[num].SetLine(int_width);

	}
};

imageBG.prototype.DisableResize= function ( num_or_id ) {
		var type = "resizable";
		var num = -1;
	if (num_or_id) {
		var num = parseInt(num_or_id);
	}
	this.Disable(type, num);
};
imageBG.prototype.DisableDrag= function ( num_or_id ) {
		var type = "draggable";
		var num = -1;
	if (num_or_id) {
		var num = parseInt(num_or_id);
	}
	this.Disable(type, num);
};

imageBG.prototype.EnableDrag= function ( num_or_id ) {
		var type = "draggable";
		var num = -1;
	if (num_or_id) {
		var num = parseInt(num_or_id);
	}
	this.Enable(type, num);
};
imageBG.prototype.EnableDrag= function ( num_or_id ) {
		var type = "resizable";
		var num = -1;
	if (num_or_id) {
		var num = parseInt(num_or_id);
	}
	this.Enable(type, num);
};

imageBG.prototype.Disable = function ( type, num ) {
	var base = bases.GetContainer();
	if ( num >= 0 ) {
		$("#"+base+num).disable(type);		
	} else {
		for ( num = 1; num < this.canvases.length; num++ ) {
			$("#"+base+num).disable();		
		}
	}
};
imageBG.prototype.Enable = function ( type, num ) {
	var base = bases.GetContainer();
	if ( num >= 0 ) {
		$("#"+base+num).enable(type);		
	} else {
		for ( num = 1; num < this.canvases.length; num++ ) {
			$.enable(base+num, type);		
		}
	}
};
imageBG.prototype.Freeze = function ( ) {
	var base = bases.GetContainer();
  var num;
  for ( num = 1; num < this.canvases.length; num++ ) {
    var container_name = this.canvases[num].container_id;
    var container = $("#"+container_name);
    container.draggable("disable");		//draggableDisable
    container.resizable("disable");
    removePretty(container_name, this);
  }
};
imageBG.prototype.Defrost = function ( ) {
	var base = bases.GetContainer();
  var num;
  for ( num = 1; num < this.canvases.length; num++ ) {
    var container_name = this.canvases[num].container_id;
    var container = $("#"+container_name);
    container.draggable("enable");		//draggableDisable
    container.resizable("enable");
    makePretty(container_name, this);
  }
};

imageBG.prototype.toPrintableString = function( ) {
	var ans = "@";
	var i;
	for ( i= 1; i<this.canvases.length;i++){
		ans += this.canvases[i].toPrintableString() + '@';
	}
	return ans;
};



imageBG.prototype.AddCanvas = function( type, left, right, top, bottom, new_color ) {

  var h = $("#"+this.id).find(".resizeHandler");
  h.each( function() {
    $(this).hide();
  });

	if ( !new_color ) { new_color = 'red'; };
	
  
  
	var num = this.counter++;
	var new_container = bases.GetContainer() + num;
	var new_canvas = bases.GetCanvas() + num;

	var copyMe = $('#' + bases.GetContainer()).clone(true).attr("id",new_container).removeClass("hidden");
  copyMe.find("canvas").attr("id", new_canvas).end();
  copyMe.appendTo($("#"+this.id));
  $("<option>" + ShapeType.Names[type] + " " + num + "</option>").val(new_canvas).appendTo("#which-one");

	this.canvases[num] = new ShapeCanvas(new_color, type, new_container, new_canvas, left, right, top, bottom);
	var parent = this;
  
  var resize_with = ShapeType.GetHandles(type);
  
/*  for( name in resize_with ) {
    $("#"+new_container).find(resize_with[name]).removeClass('hidden');
  }*/

 $('#'+new_container).draggable({
    containment: 'parent',
    cursor: 'move',
    handle: $("#"+bases.GetCanvas()+num),
 	  cancel: '.resizeHandler',
    opacity: .7,
    stop: function() {
      parent.ChangeCanvas(new_container); 
    }
  }).resizable({
    containment: 'parent',
//		handles: resize_with,
    /*handles: {
			e: '.resizeE',
			s: '.resizeS',
			se: '.resizeSE'
    },*/
    handle: '.resizeHandler', 
//		transparent: false, 
	//	proxy: 'proxy',
		stop: function(e, ui) { 
			//parent.ChangeCanvas(new_container, ui) 
			parent.handleResize(e, ui, new_container) 
		}
	});
	/*
  $('#'+new_container).resizable({
    containment: 'parent',

	//	name: new_container,
//		minWidth: 25,
//		minHeight: 25,
//		maxWidth: parent.maxWidth,
//		maxHeight: parent.maxH,
//		handles: resize_with,
    handles: {
			e: '.resizeE',
			s: '.resizeS',
			se: '.resizeSE'
    },
		transparent: true, 
		proxy: 'proxy',
		stop: function(e, ui) { handleResize(e, ui, new_container)
//			parent.ChangeCanvas(new_container); 		
		}
	});
	/*
	All callbacks (start,stop,drag) receive two arguments: The original browser event and a prepared ui object. The ui object has the following fields:

    * ui.options - options used to initialize the resizable
    * ui.axis - which handle was dragged, as a string matching the 'handles' option
    * ui.instance - the UI Resizables instance containing internal data. The most useful fields in here may be:
          o ui.instance.element - JQuery object containing the original Resizable element
          o ui.instance.helper - JQuery object with the proxy object displaying the resize
          o ui.instance.size - new size of the element, as { width, height } object*/

  makePretty(new_container, parent);
	
	return;
}

imageBG.prototype.handleResize = function (e, ui, name){
	var num = findInt(name);
	/*var new_top = 
	var new_left = 
	var new_width = 
	var new_height = 
	originalSize (height, width)
size(height, width)
position(left, top)
containerOffset(left, top)
*/

	this.canvases[num].ChangeMe( );	
};


function makePretty(new_container, parent) {
  $("#"+new_container).addClass("resizeMe");  
  $("#"+new_container).bind('mouseover', {
  bg_id : parent.id,
  container_id : new_container
  }, activateCanvas);
/**  $("#"+new_container).bind('mouseout', {
  bg_id : parent.id,
  container_id : new_container
  }, deactivateCanvas);**/

}

function removePretty(new_container, parent) {
  $("#"+new_container).removeClass("resizeMe");  
  $("#"+new_container).unbind('mouseover');
}



//imageBG.prototype.makeResizable = function( num ) {};
imageBG.prototype.ChangeCanvasColor = function(id_or_num, new_color) {
	var num = findInt(id_or_num);
	this.canvases[num].SetColor(new_color);
	this.Draw(num);
};
imageBG.prototype.Draw = function( id_or_num ) {		// optional parameter
	if (id_or_num) {
		var num = parseInt(id_or_num);
		this.canvases[num].Draw();
	} else {
		for ( num = 1; num < this.canvases.length; num++ ) {
			this.canvases[num].Draw();
		}
	}
};
imageBG.prototype.toJSON = function( image_uri ) {
	var image = $("#"+this.id);
	var image_name = image_uri.substring(image_uri.lastIndexOf('/')+1);
	var i = image_name.lastIndexOf('.');
	var imagetype = image_name.substring(i+1);
	var all = new Array();

	for(i = 1; i < this.canvases.length; i++) {
    all[i] = {
      "Shape": this.canvases[i].toJSON()
    };
	}
	
	var ans = {
		"image_with_drawings" : {
			"image" : image_name,
			"type" : imagetype,
			"drawings" : all
		}
	};
	return ans;
};
imageBG.prototype.HighlightCanvas = function (just_id) {
	var num = parseInt(just_id);
	$("#"+just_id).show('pulsate',{times: 5 });
//	this.canvases[num].addClass("highlighted");
};
/*imageBG.prototype.Rotate= function ( num_or_id ) {
	var num = parseInt(num_or_id);
	this.canvases[num].Rotate( 45 );
	this.canvases[num].Draw();
	return;
};*/

var activateCanvas = function( event ) {
  $(this).find(".resizeHandler ").show();
  var x = $("#"+event.data.container_id);
  $("#"+event.data.bg_id).one('click', function(){
    $("#"+event.data.container_id).find(".resizeHandler")
    .each(function( ){
      $(this).hide();
    });
  });
};


function StringIDBasics(resize, canvas) {
	this.resize = resize;
	this.canvas = canvas;
}
StringIDBasics.prototype.GetContainer = function() { return this.resize; };
StringIDBasics.prototype.GetCanvas = function() { return this.canvas; };

var bases = new StringIDBasics("resizeMe_", "canvas_");

